/*
 * SelectionNotAtomByIdAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

#include "Helpers/MemDebug.hpp"

#include "Actions/SelectionAction/NotAtomByIdAction.hpp"
#include "atom.hpp"
#include "log.hpp"
#include "verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "Actions/MapOfActions.hpp"


// memento to remember the state when undoing

class SelectionNotAtomByIdState : public ActionState {
public:
  SelectionNotAtomByIdState(atom* _walker) :
    walker(_walker)
  {}
  atom* walker;
};

const char SelectionNotAtomByIdAction::NAME[] = "unselect-atom-by-id";

SelectionNotAtomByIdAction::SelectionNotAtomByIdAction() :
  Action(NAME)
{}

SelectionNotAtomByIdAction::~SelectionNotAtomByIdAction()
{}

Action::state_ptr SelectionNotAtomByIdAction::performCall() {
  Dialog *dialog = UIFactory::getInstance().makeDialog();
  atom *Walker = NULL;

  dialog->queryAtom(NAME, &Walker, MapOfActions::getInstance().getDescription(NAME));

  if(dialog->display()) {
    delete dialog;
    if (World::getInstance().isSelected(Walker)) {
      World::getInstance().unselectAtom(Walker);
      return Action::state_ptr(new SelectionNotAtomByIdState(Walker));
    } else {
      return Action::success;
    }
  } else {
    delete dialog;
    return Action::failure;
  }
}

Action::state_ptr SelectionNotAtomByIdAction::performUndo(Action::state_ptr _state) {
  SelectionNotAtomByIdState *state = assert_cast<SelectionNotAtomByIdState*>(_state.get());

  if (state->walker != NULL)
    World::getInstance().selectAtom(state->walker);

  return Action::state_ptr(new SelectionNotAtomByIdState(state->walker));
}

Action::state_ptr SelectionNotAtomByIdAction::performRedo(Action::state_ptr _state){
  SelectionNotAtomByIdState *state = assert_cast<SelectionNotAtomByIdState*>(_state.get());

  if (state->walker != NULL)
    World::getInstance().unselectAtom(state->walker);

  return Action::state_ptr(new SelectionNotAtomByIdState(state->walker));
}

bool SelectionNotAtomByIdAction::canUndo() {
  return true;
}

bool SelectionNotAtomByIdAction::shouldUndo() {
  return true;
}

const string SelectionNotAtomByIdAction::getName() {
  return NAME;
}
