/*
 * SelectionNotMoleculeByIdAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

#include "Helpers/MemDebug.hpp"

#include "Actions/SelectionAction/NotMoleculeByIdAction.hpp"
#include "atom.hpp"
#include "log.hpp"
#include "verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "Actions/MapOfActions.hpp"


// memento to remember the state when undoing

class SelectionNotMoleculeByIdState : public ActionState {
public:
  SelectionNotMoleculeByIdState(molecule* _mol) :
    mol(_mol)
  {}
  molecule* mol;
};

const char SelectionNotMoleculeByIdAction::NAME[] = "select-molecule-by-id";

SelectionNotMoleculeByIdAction::SelectionNotMoleculeByIdAction() :
  Action(NAME)
{}

SelectionNotMoleculeByIdAction::~SelectionNotMoleculeByIdAction()
{}

Action::state_ptr SelectionNotMoleculeByIdAction::performCall() {
  Dialog *dialog = UIFactory::getInstance().makeDialog();
  molecule *mol = NULL;

  dialog->queryMolecule(NAME, &mol, MapOfActions::getInstance().getDescription(NAME));

  if(dialog->display()) {
    delete dialog;
    if (World::getInstance().isSelected(mol)) {
      World::getInstance().unselectMolecule(mol);
      return Action::state_ptr(new SelectionNotMoleculeByIdState(mol));
    } else {
      return Action::success;
    }
  } else {
    delete dialog;
    return Action::failure;
  }
}

Action::state_ptr SelectionNotMoleculeByIdAction::performUndo(Action::state_ptr _state) {
  SelectionNotMoleculeByIdState *state = assert_cast<SelectionNotMoleculeByIdState*>(_state.get());

  if (state->mol != NULL)
    World::getInstance().selectMolecule(state->mol);

  return Action::state_ptr(new SelectionNotMoleculeByIdState(state->mol));
}

Action::state_ptr SelectionNotMoleculeByIdAction::performRedo(Action::state_ptr _state){
  SelectionNotMoleculeByIdState *state = assert_cast<SelectionNotMoleculeByIdState*>(_state.get());

  if (state->mol != NULL)
    World::getInstance().unselectMolecule(state->mol);

  return Action::state_ptr(new SelectionNotMoleculeByIdState(state->mol));
}

bool SelectionNotMoleculeByIdAction::canUndo() {
  return true;
}

bool SelectionNotMoleculeByIdAction::shouldUndo() {
  return true;
}

const string SelectionNotMoleculeByIdAction::getName() {
  return NAME;
}
